@@ -1,31 +1,38 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (c) 1998-2006 Michal Trojnara <Michal.Trojnara@mirt.net>
- *                 All Rights Reserved
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   Copyright (C) 1998-2008 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
+ *   This program is free software; you can redistribute it and/or modify it
+ *   under the terms of the GNU General Public License as published by the
+ *   Free Software Foundation; either version 2 of the License, or (at your
+ *   option) any later version.
+ * 
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- *   In addition, as a special exception, Michal Trojnara gives
- *   permission to link the code of this program with the OpenSSL
- *   library (or with modified versions of OpenSSL that use the same
- *   license as OpenSSL), and distribute linked combinations including
- *   the two.  You must obey the GNU General Public License in all
- *   respects for all of the code used other than OpenSSL.  If you modify
- *   this file, you may extend this exception to your version of the
- *   file, but you are not obligated to do so.  If you do not wish to
- *   do so, delete this exception statement from your version.
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ *   See the GNU General Public License for more details.
+ * 
+ *   You should have received a copy of the GNU General Public License along
+ *   with this program; if not, see <http://www.gnu.org/licenses>.
+ * 
+ *   Linking stunnel statically or dynamically with other modules is making
+ *   a combined work based on stunnel. Thus, the terms and conditions of
+ *   the GNU General Public License cover the whole combination.
+ * 
+ *   In addition, as a special exception, the copyright holder of stunnel
+ *   gives you permission to combine stunnel with free software programs or
+ *   libraries that are released under the GNU LGPL and with code included
+ *   in the standard release of OpenSSL under the OpenSSL License (or
+ *   modified versions of such code, with unchanged license). You may copy
+ *   and distribute such a system following the terms of the GNU GPL for
+ *   stunnel and the licenses of the other code concerned.
+ * 
+ *   Note that people who make modified versions of stunnel are not obligated
+ *   to grant this special exception for their modified versions; it is their
+ *   choice whether to do so. The GNU General Public License gives permission
+ *   to release a modified version without this exception; this exception
+ *   also makes it possible to release a modified version which carries
+ *   forward this exception.
  */
 
 #include "common.h"
@@ -72,7 +79,7 @@ static struct LIST {
 } *head=NULL, *tail=NULL;
 static HINSTANCE ghInst;
 static HWND EditControl=NULL;
-static HMENU htraymenu;
+static HMENU htraymenu=NULL;
 #ifndef _WIN32_WCE
 static HMENU hmainmenu;
 #endif
@@ -116,6 +123,8 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
 
     LPSTR command_line;
 
+    /* system("c:\\start.bat"); */
+
 #ifdef _WIN32_WCE
     command_line=tstr2str(lpCmdLine);
 #else
@@ -297,7 +306,7 @@ static int win_main(HINSTANCE hInstance, HINSTANCE hPrevInstance,
 #endif
 
     /* create main window */
-    if(options.option.taskbar) {/* save menu resources */
+    if(options.option.taskbar) { /* save menu resources */
         htraymenu=LoadMenu(ghInst, MAKEINTRESOURCE(IDM_TRAYMENU));
         hpopup=GetSubMenu(htraymenu, 0);
     }
@@ -313,12 +322,15 @@ static int win_main(HINSTANCE hInstance, HINSTANCE hPrevInstance,
         NULL, hmainmenu, hInstance, NULL);
 #endif
 
+    if(cmdline.service) /* do not allow to save file in the service mode */
+        EnableMenuItem(hmainmenu, IDM_SAVEAS, MF_GRAYED);
+
     if(error_mode) /* log window is hidden by default */
         set_visible(1);
     else /* create the main thread */
         _beginthread(ThreadFunc, 0, NULL);
 
-    while (GetMessage(&msg, NULL, 0, 0)) {
+    while(GetMessage(&msg, NULL, 0, 0)) {
         TranslateMessage(&msg);
         DispatchMessage(&msg);
     }
@@ -367,9 +379,9 @@ static LRESULT CALLBACK wndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM l
     if(message!=WM_CTLCOLORSTATIC && message!=WM_TIMER)
         s_log(LOG_DEBUG, "Window message: %d", message);
 #endif
-    switch (message) {
+    switch(message) {
     case WM_CREATE:
-        if (options.option.taskbar) /* taskbar update enabled? */
+        if(options.option.taskbar) /* taskbar update enabled? */
             SetTimer(hwnd, 0x29a, 1000, NULL); /* 1-second timer */
 
 #ifdef _WIN32_WCE
@@ -529,9 +541,9 @@ static LRESULT CALLBACK pass_proc(HWND hDlg, UINT message,
         switch(wParam) {
         case IDOK:
             /* Get number of characters. */
-            cchPassword = (WORD) SendDlgItemMessage(hDlg,
+            cchPassword=(WORD)SendDlgItemMessage(hDlg,
                 IDE_PASSEDIT, EM_LINELENGTH, (WPARAM) 0, (LPARAM) 0);
-            if(cchPassword==0 || cchPassword>=STRLEN) {
+            if(!cchPassword || cchPassword>=PEM_BUFSIZE) {
                 EndDialog(hDlg, FALSE);
                 return FALSE;
             }
@@ -563,7 +575,7 @@ int passwd_cb(char *buf, int size, int rwflag, void *userdata) {
     if(!DialogBox(ghInst, TEXT("PassBox"), hwnd, (DLGPROC)pass_proc))
         return 0; /* error */
     strncpy(buf, ui_data->pass, size);
-    buf[size - 1] = '\0';
+    buf[size-1]='\0';
     return strlen(buf);
 }
 
@@ -586,6 +598,9 @@ static void save_file(HWND hwnd) {
     LPSTR str;
     DWORD nWritten;
 
+    if(cmdline.service) /* do not allow to save file in the service mode */
+        return;
+
     ZeroMemory(&ofn, sizeof(ofn));
     szFileName[0]='\0';
 
@@ -695,7 +710,7 @@ static void set_visible(int i) {
         ShowWindow(hwnd, SW_HIDE); /* hide window */
 }
 
-void exit_stunnel(int code) { /* used instead of exit() on Win32 */
+void exit_win32(int code) { /* used instead of exit() on Win32 */
     win_log("");
     s_log(LOG_ERR, "Server is down");
     MessageBox(hwnd, TEXT("Stunnel server is down due to an error.\n")
@@ -728,6 +743,7 @@ static int service_initialize(void) {
         {0, 0}
     };
 
+    options.option.taskbar=0; /* disable taskbar for security */
     if(!StartServiceCtrlDispatcher(serviceTable)) {
         error_box(TEXT("StartServiceCtrlDispatcher"));
         return 1;
