@@ -1,31 +1,38 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (c) 1998-2006 Michal Trojnara <Michal.Trojnara@mirt.net>
- *                 All Rights Reserved
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   Copyright (C) 1998-2008 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
+ *   This program is free software; you can redistribute it and/or modify it
+ *   under the terms of the GNU General Public License as published by the
+ *   Free Software Foundation; either version 2 of the License, or (at your
+ *   option) any later version.
+ * 
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- *   In addition, as a special exception, Michal Trojnara gives
- *   permission to link the code of this program with the OpenSSL
- *   library (or with modified versions of OpenSSL that use the same
- *   license as OpenSSL), and distribute linked combinations including
- *   the two.  You must obey the GNU General Public License in all
- *   respects for all of the code used other than OpenSSL.  If you modify
- *   this file, you may extend this exception to your version of the
- *   file, but you are not obligated to do so.  If you do not wish to
- *   do so, delete this exception statement from your version.
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ *   See the GNU General Public License for more details.
+ * 
+ *   You should have received a copy of the GNU General Public License along
+ *   with this program; if not, see <http://www.gnu.org/licenses>.
+ * 
+ *   Linking stunnel statically or dynamically with other modules is making
+ *   a combined work based on stunnel. Thus, the terms and conditions of
+ *   the GNU General Public License cover the whole combination.
+ * 
+ *   In addition, as a special exception, the copyright holder of stunnel
+ *   gives you permission to combine stunnel with free software programs or
+ *   libraries that are released under the GNU LGPL and with code included
+ *   in the standard release of OpenSSL under the OpenSSL License (or
+ *   modified versions of such code, with unchanged license). You may copy
+ *   and distribute such a system following the terms of the GNU GPL for
+ *   stunnel and the licenses of the other code concerned.
+ * 
+ *   Note that people who make modified versions of stunnel are not obligated
+ *   to grant this special exception for their modified versions; it is their
+ *   choice whether to do so. The GNU General Public License gives permission
+ *   to release a modified version without this exception; this exception
+ *   also makes it possible to release a modified version which carries
+ *   forward this exception.
  */
 
 #include "common.h"
@@ -79,15 +86,15 @@ static void init_compression(void) {
         break;
     default:
         s_log(LOG_ERR, "INTERNAL ERROR: Bad compression method");
-        exit(1);
+        die(1);
     }
     if(!cm || cm->type==NID_undef) {
         s_log(LOG_ERR, "Failed to initialize %s compression method", name);
-        exit(1);
+        die(1);
     }
     if(SSL_COMP_add_compression_method(id, cm)) {
         s_log(LOG_ERR, "Failed to add %s compression method", name);
-        exit(1);
+        die(1);
     }
     s_log(LOG_INFO, "Compression enabled using %s method", name);
 }
@@ -235,7 +242,7 @@ void open_engine(const char *name) {
     engine_initialized=0;
     if(!engines[current_engine]) {
         sslerror("ENGINE_by_id");
-        exit(1);
+        die(1);
     }
 }
 
@@ -250,7 +257,7 @@ void ctrl_engine(const char *cmd, const char *arg) {
         s_log(LOG_DEBUG, "Executing engine control command %s", cmd);
     if(!ENGINE_ctrl_cmd_string(engines[current_engine], cmd, arg, 0)) {
         sslerror("ENGINE_ctrl_cmd_string");
-        exit(1);
+        die(1);
     }
 }
 
@@ -277,11 +284,11 @@ static void init_engine() {
             sslerror("ENGINE_init");
         else
             s_log(LOG_ERR, "Engine %d not initialized", current_engine+1);
-        exit(1);
+        die(1);
     }
     if(!ENGINE_set_default(engines[current_engine], ENGINE_METHOD_ALL)) {
         sslerror("ENGINE_set_default");
-        exit(1);
+        die(1);
     }
     s_log(LOG_DEBUG, "Engine %d initialized", current_engine+1);
 }
